unit Editor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, Printers, ClipBrd;

const
  EM_FINDTEXT  = WM_USER + 56;
  FT_MATCHCASE = 4;
  FT_WHOLEWORD = 2;
  
type
  TCharRange = record
    cpMin: Longint;
    cpMax: LongInt;
  end;

  TFindText = record
    chrg: TCharRange;
    lpstrText: PChar;
  end;

  TEditorForm = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    FileMenu: TMenuItem;
    item_Save: TMenuItem;
    item_fmSep0: TMenuItem;
    item_Print: TMenuItem;
    item_PrintSetup: TMenuItem;
    item_fmSep1: TMenuItem;
    item_Exit: TMenuItem;
    item_About: TMenuItem;
    EditMenu: TMenuItem;
    item_Undo: TMenuItem;
    item_emSep0: TMenuItem;
    item_Cut: TMenuItem;
    item_Copy: TMenuItem;
    item_Paste: TMenuItem;
    item_Clear: TMenuItem;
    item_emSep1: TMenuItem;
    item_SelectAll: TMenuItem;
    item_UpperCase: TMenuItem;
    item_LowerCase: TMenuItem;
    SearchMenu: TMenuItem;
    item_Find: TMenuItem;
    item_Next: TMenuItem;
    item_Previous: TMenuItem;
    FindDlg: TFindDialog;
    PrinterSetupDlg: TPrinterSetupDialog;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
{ File Menu }
    procedure FileMenuUpdates(Sender: TObject);
    procedure item_AboutClick(Sender: TObject);
    procedure item_ExitClick(Sender: TObject);
    procedure item_PrintSetupClick(Sender: TObject);
    procedure item_PrintClick(Sender: TObject);
    procedure item_SaveClick(Sender: TObject);
{ Edit Menu }
    procedure EditMenuUpdates(Sender: TObject);
    procedure item_UndoClick(Sender: TObject);
    procedure item_CutClick(Sender: TObject);
    procedure item_CopyClick(Sender: TObject);
    procedure item_PasteClick(Sender: TObject);
    procedure item_ClearClick(Sender: TObject);
    procedure item_SelectAllClick(Sender: TObject);
    procedure item_UpperCaseClick(Sender: TObject);
    procedure item_LowerCaseClick(Sender: TObject);
{ Search Menu }
    procedure SearchMenuUpdates(Sender: TObject);
    procedure item_FindClick(Sender: TObject);
    procedure item_NextClick(Sender: TObject);
    procedure item_PreviousClick(Sender: TObject);
    procedure FindDlgFind(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  EditorForm: TEditorForm;

implementation

uses Main;

{$R *.DFM}

procedure TEditorForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

{ File Menu }

procedure TEditorForm.FileMenuUpdates(Sender: TObject);
begin
  item_Save.Enabled := Memo1.Modified;
  item_Print.Enabled := Printer.Printers.Count > 0;
  item_PrintSetup.Enabled := Printer.Printers.Count > 0;  
end;

procedure TEditorForm.item_AboutClick(Sender: TObject);
begin
  MainFormSysEdit.FileAbout(MainFormSysEdit.item_About);
end;

procedure TEditorForm.item_ExitClick(Sender: TObject);
begin
  MainFormSysEdit.FileExit(MainFormSysEdit.item_Exit);
end;

procedure TEditorForm.item_PrintSetupClick(Sender: TObject);
begin
  PrinterSetupDlg.Execute;
end;

procedure TEditorForm.item_PrintClick(Sender: TObject);
begin
  Application.ProcessMessages;
  Printer.Canvas.Font.Name := 'Courier New';
  Printer.Title := Format('SysEdit  %s', [EditorForm.Caption]);
  Printer.BeginDoc;
  Printer.Canvas.TextOut(0, 0, Memo1.Lines.Text);
  Printer.EndDoc;
end;

procedure TEditorForm.item_SaveClick(Sender: TObject);
begin
  Memo1.Lines.SaveToFile(EditorForm.Caption);
  Memo1.Modified := False;
end;

{ Edit Menu }

procedure TEditorForm.EditMenuUpdates(Sender: TObject);
begin
  item_Undo.Enabled := WordBool(Memo1.Perform(EM_CANUNDO, 0, 0));
  item_Cut.Enabled := Memo1.SelLength <> 0;
  item_Copy.Enabled := Memo1.SelLength <> 0;
  item_Paste.Enabled := Clipboard.HasFormat(CF_TEXT);
  item_Clear.Enabled := Memo1.SelLength <> 0;
  item_UpperCase.Enabled := Memo1.SelLength <> 0;
  item_LowerCase.Enabled := Memo1.SelLength <> 0;  
end;

procedure TEditorForm.item_UndoClick(Sender: TObject);
begin
  Memo1.Perform(EM_UNDO, 0, 0);
end;

procedure TEditorForm.item_CutClick(Sender: TObject);
begin
  Memo1.CutToClipboard;
end;

procedure TEditorForm.item_CopyClick(Sender: TObject);
begin
  Memo1.CopyToClipboard;
end;

procedure TEditorForm.item_PasteClick(Sender: TObject);
begin
  Memo1.PasteFromClipboard;
end;

procedure TEditorForm.item_ClearClick(Sender: TObject);
begin
  Memo1.ClearSelection;
end;

procedure TEditorForm.item_SelectAllClick(Sender: TObject);
begin
  Memo1.SelectAll;
end;

procedure TEditorForm.item_UpperCaseClick(Sender: TObject);
begin
  Memo1.SelText := AnsiUpperCase(Memo1.SelText);
end;

procedure TEditorForm.item_LowerCaseClick(Sender: TObject);
begin
  Memo1.SelText := AnsiLowerCase(Memo1.SelText);
end;

{ Search Menu }

procedure TEditorForm.SearchMenuUpdates(Sender: TObject);
begin
  item_Next.Enabled := FindDlg.FindText <> '';
  item_Previous.Enabled := FindDlg.FindText <> '';
end;

procedure TEditorForm.item_FindClick(Sender: TObject);
begin
  FindDlg.Execute;
end;

procedure TEditorForm.item_NextClick(Sender: TObject);
begin
  { not yet implemented }
end;

procedure TEditorForm.item_PreviousClick(Sender: TObject);
begin
  { not yet implemented }
end;

procedure TEditorForm.FindDlgFind(Sender: TObject);
var
  Find: TFindText;
  Flags: LongInt;
begin
  with Find.chrg do
  begin
    cpMin := 0;
    cpMax := cpMin + Length(FindDlg.FindText);
  end;
  Flags := 0;
  if frWholeWord in FindDlg.Options then Flags := Flags or FT_WHOLEWORD;
  if frMatchCase in FindDlg.Options then Flags := Flags or FT_MATCHCASE;
  Find.lpstrText := PChar(FindDlg.FindText);
  if SendMessage(Memo1.Handle, EM_FINDTEXT, Flags, LongInt(@Find)) <> 0 then
    FindDlg.CloseDialog;
end;

end.
